#include    "platform.h"
#include    "uart.h"

unsigned char uart_buffer[25];
unsigned char uabuffptr;
unsigned char uatype;



/* ==========================================================================
   all texts fo uart menu
*/
const char *textuart[]={
"SPINET - Small web server" "\xD"                               // start CPU
"Version: 1.00 07.09.2007" "\xD"
"http://pandatron.info" "\xD"
"Copyright (c) Zdenek Novotny 2007" "\xD"
"" "\xD"
"I am ready..." "\xD",
"  *** SERVISNI MENU ***" "\xD"                                   // uvodni menu
"\xD"
"Zvolte jednu z nasledujich moznosti a stisknete Enter:" "\xD"
"\xD"
"0 - restart" "\xD"
"1 - zmena IP adresy" "\xD"
"2 - zmena MAC adresy" "\xD"
"3 - info" "\xD"
"\xD"
"? ",
"\xD" "Zadejte novou IP adresu, oddelenou teckami, a stisknete Enter..." "\xD"           // change IP
"priklad: 192.168.0.10" "\xD"
"\xD"
"? ",
"\xD" "Zadejte novou MAC adresu, oddelenou pomlckami, a stisknete Enter..." "\xD"        // change MAC
"priklad: AB-BC-6F-55-1C-C2" "\xD"
"\xD"
"? ",
"\xD" "restartuji..." "\xD"                                                              // restart
"\xD",
"\xD" "zruseno" "\xD"                                                                    // storno
"\xD",
"\xD" "ulozeno" "\xD"                                                                         // storno
"Provedena zmena se projevi az po restartu!" "\xD"
"\xD"};



/* ==========================================================================
   UART refresh
   - test receiver and read
*/
void uart_refresh (void) {
	if (RCIF) {
		if(uart_read()) {
			uart_decode();
		}
	}
}



/* ==========================================================================
   decode uart buffer
*/
void uart_decode (void) {
	unsigned char chvald;

	switch(uatype) {
		case 0:                                           // in master menu
		  switch(uart_buffer[0]) {
				case 0x30:                                    // restart
				  uart_send_text(4);
				  while(1);
				break;

				case 0x31:                                    // charge IP
				  uart_send_text(2);
				  uatype = 1;
				break;

				case 0x32:                                    // charge MAC
				  uart_send_text(3);
				  uatype = 2;
				break;

				case 0x33:                                    // info
				  uart_send_text(0);
				  uatype = 0;
				break;

				default:                                      // master menu
				  uart_send_text(1);
				break;
			}
		break;

		case 1:                                           // change IP
		  if (uart_buffer[0] == 0xD) {
				uart_send_text(5);
			} else {
				chvald = 0;
				eeprom_write(8, tdec_to_value(uart_buffer[chvald], uart_buffer[chvald + 1], uart_buffer[chvald + 2], &chvald));
				eeprom_write(9, tdec_to_value(uart_buffer[chvald], uart_buffer[chvald + 1], uart_buffer[chvald + 2], &chvald));
				eeprom_write(10, tdec_to_value(uart_buffer[chvald], uart_buffer[chvald + 1], uart_buffer[chvald + 2], &chvald));
				eeprom_write(11, tdec_to_value(uart_buffer[chvald], uart_buffer[chvald + 1], uart_buffer[chvald + 2], &chvald));
		    uart_send_text(6);
			}
			uatype = 0;
		break;

		case 2:                                           // change MAC
		  if (uart_buffer[0] == 0xD) {
				uart_send_text(5);
			} else {
		    eeprom_write(0, thex_to_value(uart_buffer[0], uart_buffer[1]));
		    eeprom_write(1, thex_to_value(uart_buffer[3], uart_buffer[4]));
		    eeprom_write(2, thex_to_value(uart_buffer[6], uart_buffer[7]));
		    eeprom_write(3, thex_to_value(uart_buffer[9], uart_buffer[10]));
		    eeprom_write(4, thex_to_value(uart_buffer[12], uart_buffer[13]));
		    eeprom_write(5, thex_to_value(uart_buffer[15], uart_buffer[16]));
		    uart_send_text(6);
			}
			uatype = 0;
		break;

		default:
		  uatype = 0;
		break;
	}
	uabuffptr = 0;
}



/* ==========================================================================
   send one text
*/
void uart_send_text (unsigned char ttype) {
  unsigned char chur = 0;

  while (textuart[ttype][chur]) {
    uart_send(textuart[ttype][chur]);
    chur++;
	}
}



/* ==========================================================================
   read char from uart rec. to buffer array
*/
unsigned char uart_read (void) {
  uart_buffer[uabuffptr] = RCREG;

  if (uart_buffer[uabuffptr] == 0x08) {               // Backspace
    if (uabuffptr) uabuffptr--;
    return 0;
	}

  if (uart_buffer[uabuffptr] == 0x0D) {               // Enter is end char
		return 1;
	} else {
		if (++uabuffptr == 25) uabuffptr = 0;
		return 0;
	}
}



/* ==========================================================================
   uart initialization
   - and send init. text
*/
void uart_init (void) {
  SPBRG = 64;                                         //9k6
  TXSTA = 0x20;
  RCSTA = 0x90;

  uart_send_text(0);
}



/* ==========================================================================
   send char
   - send one byte (value) to uart
*/
void uart_send (unsigned char value) {
  while (!TRMT);
  TXREG = value;
}



/* ==========================================================================
   convert hexa-text to value
   - for MAC address
*/
unsigned char thex_to_value(unsigned char inva, unsigned char invb) {
	unsigned char chval, chvalb;

	switch (inva) {
		case 'A':
		case 'a':
		  chval = 0x0A;
		break;

		case 'B':
		case 'b':
		  chval = 0x0B;
		break;

		case 'C':
		case 'c':
		  chval = 0x0C;
		break;

		case 'D':
		case 'd':
		  chval = 0x0D;
		break;

		case 'E':
		case 'e':
		  chval = 0x0E;
		break;

		case 'F':
		case 'f':
		  chval = 0x0F;
		break;

		default:
		  chval = inva - 0x30;
		break;
	}

	switch (invb) {
		case 'A':
		case 'a':
		  chvalb = 0x0A;
		break;

		case 'B':
		case 'b':
		  chvalb = 0x0B;
		break;

		case 'C':
		case 'c':
		  chvalb = 0x0C;
		break;

		case 'D':
		case 'd':
		  chvalb = 0x0D;
		break;

		case 'E':
		case 'e':
		  chvalb = 0x0E;
		break;

		case 'F':
		case 'f':
		  chvalb = 0x0F;
		break;

		default:
		  chvalb = inva - 0x30;
		break;
	}

	return (chvalb | (chval << 4));
}



/* ==========================================================================
   convert text number to value
   - for IP address
*/
unsigned char tdec_to_value (unsigned char chvala, unsigned char chvalb, unsigned char chvalc, unsigned char *pind) {
	unsigned char chval = 0;

  if (chvala == '.' || chvala == 0xD || chvala == 0xA) {
		*pind += 1;
		return 0;
	}

  if (chvalb == '.' || chvalb == 0xD || chvalb == 0xA) {
		*pind += 2;
		return (chvala - 0x30);
	}

  if (chvalc == '.' || chvalc == 0xD || chvalc == 0xA) {
		*pind += 3;
		chval = (chvala - 0x30) * 10;
		return (chval + (chvalb - 0x30));
	}

  chval += (chvala - 0x30) * 100;
  chval += (chvalb - 0x30) * 10;
	chval += chvalc - 0x30;
	*pind += 4;

	return chval;
}



