#include    "platform.h"
#include    "udp.h"

#include    "enc28j60.h"
#include    "eth.h"
#include    "ip.h"



/* ==========================================================================
   UDP receive and decode
*/
void udp (unsigned char *buff) {
  struct UDP_header *udp;
  udp = (struct UDP_header *)&buff[34];

  switch (udp->UDP_destPort) {
    case UDP_NAMSER:                                       // name service
      name_service (buff);
    break;

    case UDP_WOL:                                          // wake on lan
      udp_wol (buff);
    break;
  }
}



/* ==========================================================================
   UDP generate and send
*/
void udp_send (unsigned int len, unsigned char *buff) {
  unsigned int tempport, sum;
  struct UDP_header *udp;
  unsigned int txbuf;
  udp = (struct UDP_header *)&buff[34];

  eth(buff);                                                    // eth header

  buff[16] = ((IP_UDP_HEADERLENGHT+len) & 0xFF00) >> 8;         // total length
  buff[17] = ((IP_UDP_HEADERLENGHT+len) & 0x00FF);

  ip(buff);                                                     // ip header

  tempport = udp->UDP_destPort;                                 // swap udp source and destination port
  udp->UDP_destPort = udp->UDP_sourcePort;
  udp->UDP_sourcePort = tempport;

  udp->UDP_lenght_h = ((UDP_HEADERLENGHT+len) & 0xFF00) >> 8;   // udp header length
  udp->UDP_lenght_l = ((UDP_HEADERLENGHT+len) & 0x00FF);

  udp->UDP_checksum = 0;                                        // checksum
  sum = checksum (&buff[26], TYPE_UDP + UDP_HEADERLENGHT + len, 16 + len);
  udp->UDP_checksum = ((sum & 0xFF00) >> 8)|((sum & 0x00FF) << 8);

  if (!nic_tx_alloc(&txbuf)) return;                            // write buffer to chip
  netbuf_wr_seek(txbuf, NB_SEEK_SET);
  netbuf_write(buff, IP_UDP_HEADERLENGHT + ETH_HEADERLENGHT + len);
  netbuf_close();

  nic_tx(IP_UDP_HEADERLENGHT + ETH_HEADERLENGHT + len);         // send

}



/* ==========================================================================
   UDP - name service
*/
void name_service (unsigned char *buff) {
  unsigned char namecount, namecountso;
  if (buff[44] == 0x01 && buff[45] == 0x10 && buff[47] == 0x01) {
    namecount = 54;
    namecountso = 0;
    while (namecount != 69 && myname[namecountso] == buff[namecount]) {  // compare name
      namecount++;
      namecountso++;
    }
    if (namecount == 69) {                                      // name is ok
      buff[44] = 0x85;                                          // generate respons
      buff[45] = 0x00;
      buff[47] = 0x00;
      buff[49] = 0x01;

      buff[92] = 0x00;
      buff[93] = 0x04;
      buff[94] = 0x93;
      buff[95] = 0xE0;
      buff[96] = 0x00;
      buff[97] = 0x06;
      buff[98] = 0x00;
      buff[99] = 0x00;
      buff[100] = myip[0];
      buff[101] = myip[1];
      buff[102] = myip[2];
      buff[103] = myip[3];

      udp_send(62,buff);                                        // send
    }
  }
}



/* ==========================================================================
   UDP - Wake On Line
*/
void udp_wol (unsigned char *buff) {
  if (buff[48] == mymac[0] && buff[49] == mymac[1] && buff[50] == mymac[2] && buff[51] == mymac[3] && buff[52] == mymac[4] && buff[53] == mymac[5]) {

    // do not use

  }
}



